/* gmatrix - combine Game of Life output from gfind or a similar program.
 * This is a quick and dirty program.
 * It contains potential buffer overruns, etc.
 * Jason Summers, 9/2000
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

int maxwidth  = 370;
int maxheight = 1200;
int xspacing  = 5;
int yspacing  = 5;

int patbuf[400][100];           // row, col

int ypos,xpos;
int filenum;
FILE *outfile;
char filenamebase[100];
char rulestring[100];
int maxysize;

void clearpatbuf(void)
{
	int i,j;
	for(j=0;j<200;j++)
		for(i=0;i<100;i++)
			patbuf[j][i]=0;
}


void opennewfile(void)
{
	char fn[100];
	if(outfile) fclose(outfile);
	sprintf(fn,"%s-%02d.lif",filenamebase,filenum);
	filenum++;

	outfile=fopen(fn,"w");
	if(!outfile) {
		printf("can't write %s\n",fn);
		exit(2);
	}
	xpos=0; ypos=0;
	printf("Writing file %s...\n",fn);

	fprintf(outfile,"#r %s",rulestring);  //includes eoln
}

int addpat(int xsize, int ysize)
{
	int i,j;
	static char fbuf[100];

	if(xpos+xsize > maxwidth && xpos>0) {
		xpos=0;
		ypos += maxysize+yspacing;
		maxysize=0;
	}
	if(ypos+ysize > maxheight && ypos>0) {
		opennewfile();
	}
	if(!outfile) {
		opennewfile();
	}

	if(ysize > maxysize) maxysize=ysize;

	fprintf(outfile,"#P %d %d\n",xpos,ypos);
	for(j=0;j<ysize;j++) {
		for(i=0;i<xsize;i++) {
			fbuf[i]= patbuf[j][i] ? 'o':'.';
		}
		fbuf[xsize]='\0';
		fprintf(outfile,"%s\n",fbuf);
	}

	xpos += xsize + xspacing;

	return 1;
}


int run_it(char *infn)
{
	FILE *f;
	int maxrow,maxcol,row,col;
	int count,i,k;
	char buf[500];
	char *rule;


	maxrow=0;
	maxcol=0;

	row=0;
	col=0;

	f=fopen(infn,"r");
	if(!f) {
		printf("can't read %s\n",infn);
		exit(2);
	}

	while(fgets(buf,500,f)) {
		if(buf[0]=='x') {
			if(!rulestring[0]) {
				rule=strchr(buf,'B');
				if(rule) {
					strcpy(rulestring,rule);
				}
			}
			continue;
		}
		if(buf[0]=='b' || buf[0]=='o' || buf[0]=='$' || buf[0]=='!' || (buf[0]>='0' && buf[0]<='9')) {
			count=1;
			for(i=0;buf[i];) {
				if(buf[i]=='o' || buf[i]=='b') {
					for(k=0;k<count;k++) {
						patbuf[row][col+k] = (buf[i]=='o')?1:0;
						if(row>maxrow) maxrow=row;
						if(col+k>maxcol) maxcol=col+k;
					}
					col+=count;
					count=1;
					i++;
				}
				else if(buf[i]=='$' || buf[i]=='!') {
					row+=count;
					col=0;
					count=1;
					i++;
				}
				else if(buf[i]>='0' && buf[i]<='9') {
					count=atoi(&buf[i]);
					while(buf[i]>='0' && buf[i]<='9') { i++; }
				}
				else {
					i++;
				}

			}
			
		}
		if(strchr(buf,'!')) {
			addpat(maxcol+1,maxrow+1);
			clearpatbuf();

			maxcol=0;
			maxrow=0;
			col=0;
			row=0;
			
		}
	}

	fclose(f);
	return 1;
}


int main(int argc,char **argv)
{
	int i;

	strcpy(filenamebase,"");

	for(i=1;i<argc;i++) {
		if(argv[i][0]=='-') {
			switch(argv[i][1]) {
			case 'w': case 'W':
				maxwidth=atoi(&argv[i][2]);
				break;
			case 'h': case 'H':
				maxheight=atoi(&argv[i][2]);
				break;
			}
		}
		else {
			strcpy(filenamebase,argv[i]);
		}
	}

	if(!strlen(filenamebase)) {
		printf("usage: gmatrix [-wMAXWIDTH] [-hMAXHEIGHT] base-file-name\n");
		printf("  example:  gmatrix -w500 -h1500 foo\n");
		printf("  -> will create files named foo-00.lif, foo-01.lif, etc.\n");
		exit(1);
	}


	filenum=0;
	outfile=NULL;
	maxysize=0;
	strcpy(rulestring,"");

	clearpatbuf();
	run_it(argv[1]);
	if(outfile) fclose(outfile);

	return 0;
}
